import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect
from turbogears import identity, validators
from sqlobject import sqlbuilder

from luca.model import conn, notfound, sqlerror, denied
from luca import model, mylib, accountlib

from luca.twolevel_table import twolevel_table

class charts_accounts(twolevel_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validatorz = {"id": validators.Int(if_invalid=0), 
		  "chart": validators.Int(if_invalid=0),   # mother
		  "xclass": validators.Int(if_invalid=0), 
		  "stdjournal": validators.Int(if_invalid=0), 
        	  "code": validators.UnicodeString(if_invalid=u""),
        	  "parent": validators.UnicodeString(if_invalid=u""),
        	  "description": validators.UnicodeString(if_invalid=u""),
        	  "smalldesc": validators.UnicodeString(if_invalid=u""),
		  "inactive": validators.Int(if_invalid=0),
		  "mobile": validators.Int(if_invalid=0)}

    validatorz_mother = {"mother": validators.Int(if_invalid=0), "id": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "chart": 0, "xclass": 0, "stdjournal": 1, "code": "", "parent": "",\
			 "description": "",  "smalldesc": "", "aggregate": 0, "mobile": 0, "inactive": 0}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	# WARNING WARNING WARNING: other test data and fixup methods assume that account mask === 9.99!
	self.sample_data = [ {"chart": 1, "xclass": 1, "stdjournal": 1, "code": "1", "parent": "", "aggregate": 1, "inactive": 0, "mobile": 0 },  \
			     {"chart": 1, "xclass": 2, "stdjournal": 1, "code": "2", "parent": "", "aggregate": 1, "inactive": 0, "mobile": 0 },  \
			     {"chart": 1, "xclass": 1, "stdjournal": 1, "code": "1.11", "parent": "1", "aggregate": 0, "inactive": 0, "mobile": 1 },  \
			     {"chart": 1, "xclass": 2, "stdjournal": 1, "code": "2.11", "parent": "2", "aggregate": 0, "inactive": 0, "mobile": 1 },  \
			     {"chart": 1, "xclass": 3, "stdjournal": 1, "code": "3", "parent": "", "aggregate": 1, "inactive": 0, "mobile": 0 },  \
			     {"chart": 1, "xclass": 4, "stdjournal": 1, "code": "4", "parent": "", "aggregate": 1, "inactive": 0, "mobile": 0 },  \
			     {"chart": 1, "xclass": 5, "stdjournal": 1, "code": "5", "parent": "", "aggregate": 1, "inactive": 0, "mobile": 0 },  \
			     {"chart": 1, "xclass": 3, "stdjournal": 1, "code": "3.11", "parent": "3", "aggregate": 0, "inactive": 0, "mobile": 1 },  \
			     {"chart": 1, "xclass": 4, "stdjournal": 1, "code": "4.11", "parent": "4", "aggregate": 0, "inactive": 0, "mobile": 1 },  \
			     {"chart": 1, "xclass": 5, "stdjournal": 1, "code": "5.11", "parent": "5", "aggregate": 0, "inactive": 0, "mobile": 1 },  \
			     {"chart": 1, "xclass": 2, "stdjournal": 1, "code": "2.14", "parent": "2", "aggregate": 0, "inactive": 1, "mobile": 1 },  \
			     {"chart": 1, "xclass": 2, "stdjournal": 1, "code": "2.13", "parent": "2", "aggregate": 0, "inactive": 0, "mobile": 1 },  \
			     {"chart": 1, "xclass": 2, "stdjournal": 1, "code": "2.12", "parent": "2", "aggregate": 0, "inactive": 0, "mobile": 1 } ]


	self.invalid_data = [ ("description", ""), ("stdjournal", 4444), ("stdjournal", 2), ("code", "1.1A"), ("code", "1."),  \
			      ("code", "1.23.4"), ("code", ""), ("xclass", 444), ("parent", "4.44"),  ("parent", "6"), ("chart", 999), \
			      ("code", "1 99") ]

	self.invalid_compound = [ dict(code="1", parent="1"), dict(code="3", parent="1.11", mobile=0) ]
	self.invalid_compound_new = [ ]
	self.invalid_compound_update = [ dict(chart=2) ]
	self.test_order = 10
	self.test_bump = [ ('charts', 'version', 'chart') ]

	self.unique_cols = ["code"] # assumed chart=1
	self.foreign_keys = [("charts", 1), ("stdjournals", 1), ("sys_classes", 1)]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Account not found, probably it has just been deleted"
	self.msg_motherinvalid = "Chart ID invalid (perhaps chart has just been deleted?)"
	self.show_msg_ajax = True
    	self.msg_added = "Account %d has been added"
    	self.msg_updated = "Account %d has been updated"
    	self.msg_removed = "Account %d has been removed sucessfully"
	self.msg_validation = "Invalid data, please fix"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Account can not be removed"
    	self.msg_unique = "Make sure the code is unique"
    	self.msg_legend_update = "Accounts: editing %s"
    	self.msg_legend_add = "Accounts: new account"

	self.title = u"Accounts of the chart"
	self.title_mask = u"Accounts of the charts %d - %s"

    	self.table = model.accounts
	self.mother_table = model.charts
	self.mother_field_name = "chart"

	self.formatters = {"_extra_": self.format_data}

    def test__nonunique_update_fixup(self, row): #pragma: no cover
	# ensures that code is 'valid' in relation to parent, so it will be consistent though nonunique
	# WARNING WARNING WARNING: the following code assumes that test rows use the mask 9.99!!!!!!!!
	if len(row["code"]) == 1:
		row["parent"] = ""
	else:
		row["parent"] = row["code"][0]
 
    def test__row_inactive(self, t, row): #pragma: no cover
	t.logmask = 0
	if not t.permission_explode("valid inactive", self.test__inactive, t.DONE, row[self.mother_field_name], row["id"]):
		t.fail("Failed inactive "+str(row))
	if t.xxx == 0 or t.xxx == 1:
		# keep inactive
		pass
	else:
		if not t.permission_explode("valid inactive", self.test__inactive, t.DONE, row[self.mother_field_name], row["id"]):
			t.fail("Failed inactive "+str(row))

    def test__row_mobile(self, t, row): #pragma: no cover
	t.logmask = 0
	if not t.permission_explode("valid mobile", self.test__mobile, t.DONE, row[self.mother_field_name], row["id"]):
		t.fail("Failed mobile "+str(row))

    def test__rows_loop_extra(self, t, row): #pragma: no cover
	self.test__row_mobile(t, row)
	self.test__row_inactive(t, row)

    def format_data(self, data):
	# a0 = non-breaking space (because putting a &nbsp here will not work)
	data['description'] = u"\xa0\xa0\xa0"*(accountlib.account_level(data['code']) - 1) + data['description']
	if 'code_checkdigit' not in data:
		data["code_checkdigit"] = data["code"]
		if model.cmos.get(1).accounts_checkdigit and not int(data["aggregate"]):
                        data["code_checkdigit"] = accountlib.add_checkdigit(data["code"])
	data['code'] = data['code_checkdigit']

    def row_name(self, d_row):
	chart_code = d_row["chart"]
	if isinstance(chart_code, model.charts):
		chart_code = chart_code.id
	return u"%d::%s" % (chart_code, d_row["code"])

    def perm_org(self, interm):
	return 0

    def make_title(self, mother):
	t = self.title
	if mother > 0:
		try:
			t = self.title_mask % (mother, self.mother_table.get(mother).description)
		except sqlerror:  #pragma: no cover
			pass
	return t

    def form_to_intermediate_ext(self, form_data, d):
   	d["description"] = d["description"][:40].strip()
   	d["smalldesc"] = d["smalldesc"][:20].strip()
   	d["code"] = d["code"][:20].strip().upper()
   	d["parent"] = d["parent"][:20].strip()

	# Cast boolean values into 1/0 field
	for item in ["mobile", "inactive"]:
		if d[item]:
			d[item] = 1
		else:
			d[item] = 0
	try:
		mask = model.charts.get(d["chart"]).mask
		if len(d["code"]) == len(mask):
			d["aggregate"] = 0
		else:	
			d["aggregate"] = 1
	except sqlerror, e:  #pragma: no cover
		d["aggregate"] = 1
		pass

	# WARNING: we don't know the chart yet.
    
    def intermediate_validate(self, d):
	if not d["description"]:
		return "Description must not be empty"
	if not d["code"]:
		return "Account code must not be empty"

	if not isinstance(d["stdjournal"], model.stdjournals):
		try:
			m = model.stdjournals.get(d["stdjournal"])
			if m.inactive:
				return "Standard journal is inactive, should not be used anymore"
		except notfound, e:
			return "Standard journal not found (maybe just deleted?)"

	
	if not isinstance(d["chart"], model.charts):
		try:
			chart_row = model.charts.get(d["chart"])
		except notfound, e:
			return "Parent chart not found (maybe just deleted?)"
	else:
		chart_row = d["chart"]

	if d["id"] > 0:
		# row update: make sure the mother chart has not been changed by a h4x0r-wannabe user
		if model.accounts.get(d["id"]).chart.id != chart_row.id:
			return "Tried to change account's chart (NOT ALLOWED!)"

	mask = chart_row.mask

	chk = accountlib.check_account_mask_naked(mask, d["code"], d["aggregate"])
	if chk:
		return "Account code error (%s)" % chk

	level = accountlib.account_level(d["code"])

	if (level > 1 and not d["parent"]):
		return "Parent code must not be empty"
	if (level == 1 and d["parent"]):
		return "Parent code must be empty for top-level account"

	if accountlib.account_level(d["code"]) >= accountlib.account_level(mask) and d["aggregate"]: #pragma: no cover
		# should never happen since aggregation is fixed by form_to_intermediate_ext method
		return "Highest-level accounts must not be aggregate"

	chk = ""
	if d["parent"]:
		chk = accountlib.check_account_mask_naked(mask, d["parent"], 1)
	if chk:
		return "Parent code error (%s)" % chk

	if level > 1:
		for row in model.accounts.select(sqlbuilder.AND(sqlbuilder.SQLConstant("chart_id = %d" % d["chart"]), 
						      model.accounts.q.code == d["parent"])):
			parent_row = row
			break
		else:
			return "Parent does not exist"

		if not parent_row.aggregate:
			return "Parent must be an aggregate account"
	
		if accountlib.account_level(parent_row.code) >= level:
			return "Parent aggregation level must be higher than the daughter"
	
		# note that we are *FIXING* an invalid value (intermediate_validate can do this!)
		if not isinstance(d["xclass"], model.classes) and d["xclass"] <= 0:
			d["xclass"] = parent_row.xclass.id

		if not d["aggregate"] and parent_row.xclass.id != d["xclass"]:
			return "Non-aggregate account must have the same class as its parent"
		
	if not isinstance(d["xclass"], model.classes):
		try:
			model.classes.get(d["xclass"])
		except notfound, e:
			return "Account class not found"

	if d["id"] > 0:
		# too good that account IDs are unique even across different charts...
		where = sqlbuilder.SQLConstant("account_id = %d" % d["id"])
		if model.journal.select(where).count() > 0:
			if d["aggregate"]:
				return "Account cannot be aggregate since it already has journals"
			try:
				old_row = model.accounts.get(d["id"])
			except notfound:
				return "Account has just been deleted, please cancel and try again"
			if old_row.code != d["code"]:
				return "Account code can not be changed since it already has journals"
	if d["id"] > 0:
		if self.table.select(sqlbuilder.AND(self.table.q.id != d["id"], sqlbuilder.SQLConstant("chart_id = %d" % d["chart"]), self.table.q.code == d["code"])).count() > 0:
			return self.msg_integrity + self.msg_unique
	else:
		if self.table.select(sqlbuilder.AND(sqlbuilder.SQLConstant("chart_id = %d" % d["chart"]), self.table.q.code == d["code"])).count() > 0:
			return self.msg_integrity + self.msg_unique

	return ""

    def row_to_intermediate_ext(self, row, intermediate):
	    '''Data dictionary manipulation just before editing an existing row'''
	    # Reduces sub-rows to their IDs
	    if isinstance(intermediate["chart"], model.charts):
		intermediate["chart"] = intermediate["chart"].id
	    if isinstance(intermediate["xclass"], model.classes):
		intermediate["xclass"] = intermediate["xclass"].id
	    if isinstance(intermediate["stdjournal"], model.stdjournals):
		intermediate["stdjournal"] = intermediate["stdjournal"].id

    def data_legend(self, d):
	    return str(d["chart"])+"::"+d["code"]

# INHERITED METHODS ########################################################

    def new_row_recoil(self, row_data):
	row_data["xclass"] = 0

    def trigger_update_post(self, row, id, new, form_data):
	chart = row["chart"] 
	if not isinstance(chart, model.charts):
		chart = model.charts.get(chart)
	chart.bump_version()

    def trigger_remove_post(self, row):
	chart = row["chart"] 
	if not isinstance(chart, model.charts):
		chart = model.charts.get(chart)
	chart.bump_version()

    def table_version(self, row):
	version = -1
	if "chart" in row and row["chart"] > 0:
	    version = self.mother_table.get(row["chart"]).version
	elif "mother" in row and row["mother"] > 0:
	    version = self.mother_table.get(row["mother"]).version
	return version

    def get_index_rows(self, mother):
	rows = []
	softerr = ""
	err = ""
	if mother > 0:
		try:
			try:
				chart = model.charts.get(mother)
				accounts_checkdigit = model.cmos.get(1).accounts_checkdigit
			except notfound:
				err = "Chart not found!"
			if not err:
				try:
					rows, softerr = accountlib.check_chart(chart, accounts_checkdigit)
				except notfound:
					err = "Chart is empty."
				for row in rows:
					self.format_data(row)
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e)
		if softerr:
			softerr = "Chart has errors, see highlighted accounts for error details"
	return (rows, err, softerr, {})

    def auxiliary_form_data(self, form_data):
	err = ""
	form_data["xclasses"] = [{"id": 0, "description": "(automatic)"}]
	form_data["mask"] = ""
	form_data["mask_separator"] = accountlib.mask_separator
	form_data["mask"] = "XXX.XXX"
	form_data["mobiles"] = [{"id": 0, "description": "No"}, {"id": 1, "description": "Yes"}]
	form_data["inactives"] = [{"id": 0, "description": "Active"}, {"id": 1, "description": "Inactive"}]
	form_data["mothers"] = [{"id": 0, "description": "---"}]
	for row in model.charts.select(orderBy=model.charts.q.description):
		form_data["mothers"].append({"id": row.id, "description": row.description})
	for row in model.classes.select(orderBy=model.classes.q.oid):
		form_data["xclasses"].append({"id": row.id, "description": row.description})
	if form_data["chart"] > 0:
		form_data["mask"] = model.charts.get(form_data["chart"]).mask
	return err

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.charts_accounts")
    @validate(validators=validatorz_mother)
    def index(self, mother=0, **dummy):
        return self.base_index(False, mother)

    @expose(template="luca.templates.charts_accounts_lite")
    @validate(validators=validatorz_mother)
    def lite(self, mother=0, **dummy):
        d = self.base_index(True, mother)
	return d

    @expose("json")
    @validate(validators=validatorz)
    def remove(self, id, **garbage):
	    result = self.base_remove(id)
	    return result

    @expose()
    @validate(validators=validatorz_mother)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validatorz_mother)
    def fetch(self, mother, id, **garbage):
	    return self.base_fetch(mother, id)

    @expose(template="luca.templates.charts_accounts_lite_form")
    @validate(validators=validatorz_mother)
    def lite_edit(self, mother, id, **garbage):
	   d = self.base_lite_edit(mother, id)
	   return d

    @expose("json")
    @validate(validators=validatorz)
    def update(self, **form_data):
	    result = self.base_update(form_data)
	    return result

    @expose(template="luca.templates.charts_accounts_lite_form")
    @validate(validators=validatorz)
    def lite_commit(self, **form_data):
	    d = self.base_lite_commit(form_data)
            # if program flow here, something was wrong (validation msg is inside d)
	    return d

    def toogle_mobile(self, mother, id):
	    # simulates a data fetch, receives form data
	    form_data = self.base_fetch(mother, id)
	    if form_data["err"]:
		    return form_data
	    if form_data["mobile"]:
	    	form_data["mobile"] = 0
	    else:
	    	form_data["mobile"] = 1
	    form_data["err"] = self.base_update(form_data)["err"]
	    return form_data

    def toogle_inactive(self, mother, id):
	    # simulates a data fetch, receives form data
	    form_data = self.base_fetch(mother, id)
	    if form_data["err"]:
		    return form_data
	    if form_data["inactive"]:
	    	form_data["inactive"] = 0
	    else:
	    	form_data["inactive"] = 1
	    form_data["err"] = self.base_update(form_data)["err"]
	    return form_data

    @expose("json")
    @validate(validators=validatorz_mother)
    def mobile(self, mother, id, **garbage):
	return self.toogle_mobile(mother, id)

    @expose("json")
    @validate(validators=validatorz_mother)
    def inactive(self, mother, id, **garbage):
	return self.toogle_inactive(mother, id)

