import cherrypy

import turbogears
from turbogears import controllers, expose, validate
from turbogears import identity, validators

from luca.model import cmos, conn, notfound, denied
from luca import model

from luca.simple_table import simple_table

class orgs(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
        	  "name": validators.UnicodeString(if_invalid=u""),
        	  "remarks": validators.UnicodeString(if_invalid=u""),
	          "chart": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "name": "", "remarks": "", "chart": 0}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read
	
	# data for automated tests
	self.sample_data = [{"chart": 1}, {"chart": 2}, {"chart": 2}, {"chart": 1}]
	self.invalid_data = [("name", ""), ("chart", 0)]
	self.unique_cols = []
	self.foreign_keys = [("charts", 1), ("charts", 2), ("sys_users", 2) ] # sys_users = via log 
	self.test_order = 5
	self.test_flags = ('admin_as_winner','noorg_index_allowed', 'noorg_insert_allowed')
	self.test_bump = [ ('orgs', 'version', 'id'), ('cmos', 'orgs_version', 1) ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Organization not found, probably it has just been deleted"
	self.show_msg_ajax = True
    	self.msg_added = "Organization %d has been added"
    	self.msg_updated = "Organization %d has been updated"
    	self.msg_removed = "Organization %d has been removed sucessfully"
	self.msg_validation = "Order number and name must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Organization can not be removed"
    	self.msg_cannotchangechart = "Organization already has batches/journal entries - cannot change chart!"
    	self.msg_unique = "check if the order number is unique"
    	self.msg_legend_update = "Organizations: editing organization '%s'"
    	self.msg_legend_add = "Organizations: new organization"
    	self.msg_chartnotfound = "Chart not found in database!"

    	self.table = model.orgs
	self.cmos = cmos
	self.streamlined_add = True

	self.formatters = {"_extra_": self.format_data}

    def format_data(self, data):
	if 'chart_description' not in data:
		try:
			data['chart_description'] = model.charts.get(data["chart"]).description
		except sqlerror, e:  #pragma: no cover
			data['chart_description'] = ""

    def row_name(self, d_row):
	name = ""
	if d_row["id"] > 0:
		name = u"%s (id %d)" % (d_row["name"], d_row["id"])
	else:
		name = d_row["name"]
	return name

    def form_to_intermediate_ext(self, form_data, d):
   	d["name"] = d["name"][:40].strip()
   	d["remarks"] = d["remarks"].strip()

    def row_to_intermediate_ext(self, row, intermediate):
	if type(intermediate["chart"]) is model.charts:
		intermediate["chart"] = intermediate["chart"].id
    
    def data_legend(self, intermediate):
	return unicode(intermediate["id"])+" - "+intermediate["name"]

    def intermediate_validate(self, d):
	    if not d["name"]:
		return "Description must not be empty"

	    try: 
		    model.charts.get(d["chart"])
	    except notfound:
		    return self.msg_chartnotfound

	    if d["id"] > 0:
		row = self.table.get(d["id"])
		if row.chart.id != d["chart"]:
			if model.journal.select("ORG_ID = %d" % d["id"]).count() > 0:
		    		return self.msg_cannotchangechart
	    
	    return ""
		   
    def perm_org(self, row):
	return row["id"]

    def auxiliary_form_data(self, form_data):
	form_data["charts"] = model.charts.select(orderBy=model.charts.q.description)
	
    def trigger_update_post(self, row, id, new, form_data):
	    model.cmos.bump_version("orgs")
	    self.table.get(id).bump_version()

    def trigger_remove_post(self, row):
	    model.cmos.bump_version("orgs")

    def table_version(self, row):
	return model.cmos.get_version("orgs")

    def trigger_remove_pre(self, row):
	t = model.batches.select("org_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Organization contains batch %d" % (l.id)
	t = model.journal.select("org_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Organization contains journal %d:%d" % (l.batch, l.id)
	return ""

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.orgs")
    def index(self, **dummy):
        return self.base_index(orderBy=self.table.q.name)

    @expose(template="luca.templates.orgs_lite")
    def lite(self, **dummy):
        return self.base_index(orderBy=self.table.q.name)

    @expose("json")
    @validate(validators=validators)
    def remove(self, id, **garbage):
	    result = self.base_remove(id)
	    return result

    @expose()
    @validate(validators=validators)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    d = self.base_fetch(id)
	    return d

    @expose(template="luca.templates.orgs_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	    return self.base_lite_edit(id)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    result = self.base_update(form_data)
	    return result

    @expose(template="luca.templates.orgs_lite_form")
    @validate(validators=validators)
    def lite_commit(self,  **form_data):
	    return self.base_lite_commit(form_data)

