import cherrypy

import turbogears
from turbogears import controllers, redirect, flash
from turbogears import identity

from luca.model import conn, notfound,  sqlerror, denied

from model import permission

class parentXchild(controllers.Controller):

# MODEL- AND LANGUAGE-SPECIFIC CODE & MESSAGES ######################################

    def setup(self): #pragma: no cover

# 	    self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
#	    self.msg_notfound = "User not found, probably it has just been deleted"
#           self.msg_updated = "User has been updated"
#           self.msg_validation = "Name, nickname and e-mail address must not be empty"
#           self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
#    	    self.parent_table = xuser
#    	    self.child_table = xgroup
#    	    self.cross_table = user_group
#    	    self.add_child_method= self.parent_table.addXgroup
#    	    self.del_child_method = self.parent_table.delXgroup
#    	    self.parent_name = "user_name"
#    	    self.child_name = "group_name"
#    	    self.parent_order = self.parent_table.q.user_name
#    	    self.child_order = self.child_table.q.group_name
#     	    self.no_parent = "(None)"
#     	    self.parent_children_property = "groups"

	raise StandardError("You should have overriden this!")

# BOILERPLATE CODE (INIT & ERRORS) ################################################
 
    def __init__(self, log):
        super(parentXchild, self).__init__()
	self._indexflash = None    # flash message to be shown in the next index page
	self._indexflash_err = 0
	self.setup()
    	self.log = log

    def flash_at_index(self, msg, err):
	self._indexflash = msg
	self._indexflash_err = err

    def flash(self, form_data, msg, err):
	flash(msg)
	if err:
		form_data["tg_flash_class"] = "errmsg"
	else:
		form_data["tg_flash_class"] = "infomsg"

    def sqlerr(self, exception):  #pragma: no cover
	    self.log.error("%s SQL error %s", self.__class__.__name__, str(exception.args))
	    return self.msg_unexpected



# MAIN PAGE METHODS #################################################

    def base_get_parent(self, parent_now_id, prev_err, prev_msg):

	try:
		grant = permission(self, "read", 0, 0, parent_now_id)

		order = 0
		parents = [{"id": 0, "name": "---", "selected": "1"} ]
		parent_now = None
		d_available = {}
		available = []
		employed = []
		err = ""

		# gets all available children
		for child in self.child_table.select(orderBy=self.child_order):
			order = order + 1
			d_available[child.id] = {"id": child.id, "name": getattr(child, self.child_name), "order": order}

		# gets all available parents
		for parent in self.parent_table.select(orderBy=self.parent_order):
			parent_name = getattr(parent, self.parent_name)
			parents.append({"id": parent.id, "name": parent_name, "selected": None})
			# is this the current parent?
			if parent.id == parent_now_id:
				parent_now = parent_name
				parents[-1]["selected"] = "1"
				# Deselects the "null" parent
				parents[0]["selected"] = None
				# handles children
				row = self.parent_table.get(parent.id)
				for child in getattr(row, self.parent_children_property):
					if child.id in d_available:
						# removes from available, adds to employed
						employed.append(d_available[child.id])
						del d_available[child.id]
					else:
						# unexpected, we ignore.
						pass
	except denied, e:
        	d = dict(available=[], employed=[], parent_now="", parent_now_id=0, parents=[], err=e.args[0])
		self.flash(d, e.args[0], True)
		return d

	except sqlerror, e:  #pragma: no cover
		parent_now = None
		err = self.sqlerr(e)

	# Tests for null/nonexistent parent
	if parent_now is None:
		parent_now = self.no_parent
		parent_now_id = 0

	# Converts available children dictionary to a list
	for id in d_available.keys():
		available.append(d_available[id])

	# Sorts lists
	available.sort(lambda x,y: cmp(x["order"], y["order"]))
	employed.sort(lambda x,y: cmp(x["order"], y["order"]))

        d = dict(available=available, employed=employed, parent_now=parent_now, parent_now_id=parent_now_id, parents=parents, err=err or prev_err)

	if err:
		self.flash(d, err, True)
	else:
		if prev_err:
			self.flash(d, prev_err, True)
		else:
			self.flash(d, prev_msg, False)

	return d

    
    def check(self, parent_now_id, employed, available, action):
	'''Overridable checking before update
	   	* Return non-empty string to report error
		* It is allowed to manipulat employed and available lists'''
	raise StandardError("You should have overridden this!")

    def base_update(self, **dummy):
	raise StandardError("Method is useless for this class")

    def children_update(self, parent_now_id, employed, available, action):
	err = self.check(parent_now_id, employed, available, action)
	if err:
		return err

	try:
		parent = self.parent_table.get(parent_now_id)
	except notfound, e:
	        return self.msg_notfound
	except sqlerror, e:  #pragma: no cover
		return self.sqlerr(e)
	
	try:
		grant = permission(self, "write.%s" % action, 0, 0, parent_now_id, getattr(parent, self.parent_name))
		if action == "add":
			for id in available:
				child = self.child_table.get(id)
				self.add_child_method(parent, child)
		elif action == "remove":
			for id in employed:
				child = self.child_table.get(id)
				self.del_child_method(parent, child)
		grant.done()
	except denied, e:
		return e.args[0]
	except sqlerror, e:  #pragma: no cover
		self.sqlerr(e)
		grant.fail()
		pass
	
	return ""

    def sanitize_set(self, s):
	# Sets are not completely validated by validator.Set(). This function ensures
	# that every element is a positive integer (id)
	try:
		newset = [ int(x) for x in s ]
	except ValueError:
		# bad, bad set! no donut for you!
		newset = []
	return newset

    def base_index(self, select_parent, parent_now_id, employed, available, button_add, button_del, button_select):
	err = ""
	msg = ""

	if parent_now_id > 0 and (button_add or button_del):
		employed = self.sanitize_set(employed)
		avaiable = self.sanitize_set(available)
		err = self.children_update(parent_now_id, employed, available, button_add and "add" or "remove")
		if not err:
			msg = self.msg_updated
	elif button_select:
		parent_now_id = select_parent

	return self.base_get_parent(parent_now_id, err, msg)

