import turbogears
from turbogears import controllers, redirect, flash
from turbogears import identity, validators, expose, validate

from luca.model import sqlerror, notfound, log, denied
from luca import model

from sqlobject import sqlbuilder

import mylib
import reportlib
import datetime
import time

from simple_report import simple_report

class report_balance(simple_report):

# MODEL- AND LANGUAGE-SPECIFIC CODE & MESSAGES ######################################

    validators = { \
	          "date1": validators.UnicodeString(if_invalid=u""),
	          "date2": validators.UnicodeString(if_invalid=u""),
		  "org": validators.Int(if_invalid=0),  \
	          "result": validators.Int(if_invalid=0),
		  "submitted": validators.Int(if_invalid=0) }
				 
    def setup(self):
 	   self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
           self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
           self.msg_validation = self.msg_validation_crash
	   self.msg_initialbalance = u"(Initial balance)"
	   self.title = u"Balance report"
	   self.title_trial = u"Trail balance"
	   self.title_final = u"Balance statement"
	   self.title_mask = "%s - %d %s - %s to %s"

           self.valid_requests = [ dict(org=1, result=0, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, result=1, date2="2006/01/01", date1="2006/01/31", submitted=1) ]

           self.invalid_requests = [ dict(org=222, result=0, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, result=0, date1="200", date2="2006/01/31", submitted=1), \
				dict(org=-1, result=0, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=999, result=0, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, result=0, date2="200", date1="2006/01/31", submitted=1) ]
	   self.results = [{"id": 0, "description": "Balance statement (no income/expense accounts)"}, {"id": 1, "description": "Trial balance (with income/expense accounts)"}]
	   self.vips = ["REPORTS"]

 
    def __init__(self, log):
        super(report_balance, self).__init__()
    	self.log = log


    def check(self, org, result, date1, date2):
	err = ""
	date1 = mylib.ctod(date1)
	date2 = mylib.ctod(date2)

	if date1 is None:
		err = "Initial date is invalid"
	elif date2 is None:
		err = "Final date is invalid"
	elif org <= 0:
		err = "Please choose an organization"
	else:
		if date1 > date2:
			date1, date2 = date2, date1

		try:
			org = model.orgs.get(org)
		except notfound, e:
			err = "Organization does not exist (just deleted?)"
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e)+" (SQL Error) "

	return (err, org, result, date1, date2)


    def query(self, org, result, date1, date2):
	err = ""
	rows = []
	dbsum = crsum = ""
	dbsumn = crsumn = 0
	title = self.title

	err, rorg, result, ddate1, ddate2 = self.check(org, result, date1, date2)
	if err:
		return (title, rows, dbsum, crsum, err)

	if result:
		sresult = self.title_trial
	else:
		sresult = self.title_final

	title = self.title_mask % (sresult, rorg.id, rorg.name, mylib.dtoc(ddate1), mylib.dtoc(ddate2))

	mylib.setdefault(self, "date1", date1, "")
	mylib.setdefault(self, "date2", date2, "")
	mylib.setdefault(self, "result", result, -1)
	mylib.setdefault(self, "org", org, 0)

	dresult = 1
	if result:
		dresult = 0

	try:
		raw_report = reportlib.balance_report(rorg.id, ddate1, ddate2, 0, 0, dresult, 0)
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) + "(while calculating balance)"
		return (title, rows, dbsum, crsum, err)

	if raw_report is None:
		err = "Balance not calculated - chart of accounts has problems"
		return (title, rows, dbsum, crsum, err)

	for row in raw_report:
		if not row["inbal"] and not row["db"] and not row["cr"]:
			continue
		rows.append(row)
		row["fin"] = row["inbal"] - row["db"] + row["cr"]
		if row["normal_balance"]:
			# normal balance is DB
			row["inbal"] = -row["inbal"]
			row["fin"] = -row["fin"]
		if not row["aggregate"]:
			dbsumn += row["db"]
			crsumn += row["cr"]
		row["inbal"], row["inbal_class"] = mylib.monetary_balance(row["inbal"])
		row["db"], dummy = mylib.monetary_balance(row["db"])
		row["cr"], dummy = mylib.monetary_balance(row["cr"])
		row["fin"], row["fin_class"] = mylib.monetary_balance(row["fin"])
		row["ledger_params"] = {"org": rorg.id, "date1": date1, "date2": date2, "account": row["code"], "submitted": 1}

	dbsum = mylib.monetary(dbsumn)
	crsum = mylib.monetary(crsumn)

	rows.sort(lambda x,y: cmp(x["code"], y["code"]))

	x = 0
	for row in rows:
		if row["aggregate"]:
			x = 0
			if row["level"] == 1:
				row["css_row"] = "toplevel"
			else:
				row["css_row"] = "aggregate"
		else:
			row["css_row"] = x % 2 and "odd" or "even"
			x += 1

	return (title, rows, dbsum, crsum, err) 


    def select_tables(self):
	orgs = [{"id": 0, "name": "---"}]
	err = ""

	try:
		for row in model.orgs.select(orderBy=model.orgs.q.name):
			orgs.append({"id": row.id, "name": "%d - %s" % (row.id, row.name)})
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) + "(reading organizations)"

	return (orgs, self.results[:], err)

    def perm_org(self, d):
	return d["org"]

    def page(self, org, result, date1, date2, submitted):
	err = ""
	rows = []
	dbsum = crsum = ""
	title = self.title
	if submitted:
		try:
			grant = model.permission(self, "read", org, 0)
			title, rows, dbsum, crsum, err = self.query(org, result, date1, date2)
			if err:
				grant.fail(err)
			else:
				grant.done()
		except denied, e:
			err = e.args[0]
	
	date1 = mylib.getdefault(self, "date1", date1, "")
	date2 = mylib.getdefault(self, "date2", date2, "")
	result = mylib.getdefault(self, "result", result, 0)
	org = mylib.getdefault(self, "org", org, 0)

	orgs, results, err2 = self.select_tables()

	if err or err2:
		flash(err or err2)
		err = err or err2

	return dict(title=title, org=org, date1=date1, date2=date2, results=results, result=result, \
		    orgs=orgs, rows=rows, dbsum=dbsum, crsum=crsum, tg_flash_class="errmsg", \
		    timestamp=mylib.timestamp(), err=err)

    @expose(template="luca.templates.report_balance")
    @identity.require(identity.not_anonymous())
    @validate(validators=validators)
    def index(self, org=0, result=0, date1="", date2="", submitted=0, **dummy):
	return self.page(org, result, date1, date2, submitted)

    @expose(template="luca.templates.report_balance_lite")
    @identity.require(identity.not_anonymous())
    @validate(validators=validators)
    def lite(self, org=0, result=0, date1="", date2="", submitted=0, **dummy):
	return self.page(org, result, date1, date2, submitted)

