import turbogears
from turbogears import controllers, redirect, flash
from turbogears import identity, validators, expose, validate

from luca.model import sqlerror, notfound, log, denied
from luca import model

from sqlobject import sqlbuilder

import mylib
import reportlib
import datetime
import time
import accountlib

from simple_report import simple_report

class report_cashflow(simple_report):

# MODEL- AND LANGUAGE-SPECIFIC CODE & MESSAGES ######################################

    validators = { \
	          "date1": validators.UnicodeString(if_invalid=u""),
	          "date2": validators.UnicodeString(if_invalid=u""),
		  "org": validators.Int(if_invalid=0),  \
		  "aggregation": validators.Int(if_invalid=0),  \
		  "submitted": validators.Int(if_invalid=0) }
				 
    def setup(self):
 	   self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
           self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
           self.msg_validation = self.msg_validation_crash
	   self.title = u"Cash flow statement"
	   self.title_mask = self.title + u" - %d %s - %s to %s"
 
           self.valid_requests = [ dict(org=1, aggregation=0, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, aggregation=1, date2="2006/01/01", date1="2006/01/31", submitted=1) ]

           self.invalid_requests = [ 
				dict(org=1, aggregation=1,  date1="200", date2="2006/01/31", submitted=1), \
				dict(org=1, aggregation=900, date2="2006/01/01", date1="2006/01/31", submitted=1), \
				dict(org=-1, aggregation=1,  date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=999, aggregation=1,  date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, aggregation=1,  date2="200", date1="2006/01/31", submitted=1) ]
	   self.vips = ["REPORTS"]
	   self.aggregations = [  {"id": 0, "description": "No aggregation"}, \
				{"id": 1, "description": "1 level"}, \
				{"id": 2, "description": "2 levels"}, \
				{"id": 3, "description": "3 levels"},
				{"id": 4, "description": "4 levels"},
				{"id": 5, "description": "5 levels"}  ]


    def __init__(self, log):
        super(report_cashflow, self).__init__()
    	self.log = log

    def check(self, org, date1, date2, aggregation):
	err = ""
	date1 = mylib.ctod(date1)
	date2 = mylib.ctod(date2)

	if date1 is None:
		err = "Initial date is invalid"
	elif date2 is None:
		err = "Final date is invalid"
	elif org <= 0:
		err = "Please choose an organization"
	elif aggregation < 0 or aggregation >= len(self.aggregations):
		err = "Aggregation level %d is invalid" % aggregation
	else:
		if date1 > date2:
			date1, date2 = date2, date1

		try:
			org = model.orgs.get(org)
		except notfound, e:
			err = "Organization does not exist (just deleted?)"
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e)+" (SQL Error) "

	return (err, org, date1, date2)

    def paint_children(self, pack, parent, color):
	for item in pack:
		if item["parent"] == parent["code"]:
			item["COLOR"] = color
			if item["aggregate"]:
				self.paint_children(pack, item, color)

    def query(self, org, date1, date2, aggregation):
	err = ""
	rows = []
	dbsum = crsum = ""
	dbsumn = crsumn = 0
	title = self.title

	err, rorg, ddate1, ddate2 = self.check(org, date1, date2, aggregation)
	if err:
		return (title, rows, dbsum, crsum, err)

	title = self.title_mask % (rorg.id, rorg.name, mylib.dtoc(ddate1), mylib.dtoc(ddate2))

	mylib.setdefault(self, "date1", date1, "")
	mylib.setdefault(self, "date2", date2, "")
	mylib.setdefault(self, "org", org, 0)

	max_level = accountlib.account_level(rorg.chart.mask)
	level = max_level - aggregation
	if level < 1:
		level = 1
	if level > max_level:
		level = max_level

	try:
		raw_report = reportlib.balance_report(rorg.id, ddate1, ddate2, 0, 1, 0, 0)
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) + "(while calculating incoming statement)"
		return (title, rows, dbsum, crsum, err)

	if raw_report is None:
		err = "Income statement not calculated - chart of accounts has problems"
		return (title, rows, dbsum, crsum, err)

	raw_report2 = []
	for row in raw_report:
		row["COLOR"] = 0

	dirty = 1
	while dirty and not err:
		dirty = 0	
		for row in raw_report:
			if not row["aggregate"] and row["COLOR"] == 0:
				# account not properly represented in the report
				dirty = 1
				# get the first account's parent that is <= report level
				# (may be itself if the report is not aggregate)
				parent = row
				while parent["level"] > level:
					# find the account's parent in the list
					for row2 in raw_report:
						if row2["code"] == parent["parent"]:
							parent = row2
							break
					else:
						err = "Internal error - no parent II"
						break
				# add the parent account to the report
				parent["COLOR"] = 2
				# show that all its children are properly covered now
				self.paint_children(raw_report, parent, 1)

	if err:
		return (title, rows, dbsum, crsum, err) 

	for row in raw_report:
		if not row["db"] and not row["cr"]:
			continue
		if row["COLOR"] != 2:
			continue
		rows.append(row)
		row["liq"] = row["cr"] - row["db"]
		if row["liq"] < 0:
			dbsumn += -row["liq"]
			row["db"], dummy = mylib.monetary_balance(-row["liq"])
			row["cr"] = ""
		else:
			crsumn += row["liq"]
			row["db"] = ""
			row["cr"], dummy = mylib.monetary_balance(row["liq"])
		row["ledger_params"] = {"org": rorg.id, "date1": date1, "date2": date2, "account": row["code"], "submitted": 1}

	rows.sort(lambda x,y: cmp(x["liq"], y["liq"]))

	x = 0
	for row in rows:
		row["css_row"] = x % 2 and "odd" or "even"
		x += 1

	crsum = mylib.monetary(crsumn)
	dbsum = mylib.monetary(dbsumn)

	return (title, rows, dbsum, crsum, err) 


    def select_tables(self):
	orgs = [{"id": 0, "name": "---"}]
	err = ""

	try:
		for row in model.orgs.select(orderBy=model.orgs.q.name):
			orgs.append({"id": row.id, "name": "%d - %s" % (row.id, row.name)})
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) + "(reading organizations)"

	return (orgs, self.aggregations[:], err)

    def perm_org(self, d):
	return d["org"]


    @expose(template="luca.templates.report_cashflow")
    @identity.require(identity.not_anonymous())
    @validate(validators=validators)
    def index(self, org=0, aggregation=0, date1="", date2="", submitted=0, **dummy):
	err = ""
	rows = []
	dbsum = crsum = ""
	title = self.title
	if submitted:
		try:
			grant = model.permission(self, "read", org, 0)
			title, rows, dbsum, crsum, err = self.query(org, date1, date2, aggregation)
			if err:
				grant.fail(err)
			else:
				grant.done()
		except denied, e:
			err = e.args[0]
	
	date1 = mylib.getdefault(self, "date1", date1, "")
	date2 = mylib.getdefault(self, "date2", date2, "")
	org = mylib.getdefault(self, "org", org, 0)
	aggregation = mylib.getdefault(self, "aggregation", aggregation, 0)

	orgs, aggregations, err2 = self.select_tables()

	if err or err2:
		flash(err or err2)
		err = err or err2

	return dict(title=title, org=org, date1=date1, date2=date2, orgs=orgs, aggregation=aggregation, aggregations=aggregations, rows=rows, dbsum=dbsum, crsum=crsum, tg_flash_class="errmsg", timestamp=mylib.timestamp(), err=err)

    @expose() #pragma: no cover
    def lite(self, **dummy): #pragma: no cover
	redirect("./") #pragma: no cover

