import cherrypy

import turbogears
from turbogears import controllers, redirect, flash
from turbogears import identity, validators, expose, validate

from luca.model import sqlerror, notfound, log, denied
from luca import model

from sqlobject import sqlbuilder

import mylib
import reportlib
import datetime
import time

from simple_report import simple_report

class report_journal(simple_report):

# MODEL- AND LANGUAGE-SPECIFIC CODE & MESSAGES ######################################

    validators = { \
	          "date1": validators.UnicodeString(if_invalid=u""),
	          "date2": validators.UnicodeString(if_invalid=u""),
		  "org": validators.Int(if_invalid=0),  \
		  "submitted": validators.Int(if_invalid=0) }
				 
    def setup(self):
 	   self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
           self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
           self.msg_validation = self.msg_validation_crash
	   self.msg_initialbalance = u"Initial balance"
	   self.title = u"Journal report"
	   self.title_mask = self.title + u" - %d %s - %s to %s"

           self.valid_requests = [ dict(org=1, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, date2="2006/01/01", date1="2006/01/31", submitted=1) ]

           self.invalid_requests = [ dict(org=222, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, date1="200", date2="2006/01/31", submitted=1), \
				dict(org=-1, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=999, date1="2006/01/01", date2="2006/01/31", submitted=1), \
				dict(org=1, date2="200", date1="2006/01/31", submitted=1) ]
	   self.vips = ["REPORTS"]

    def __init__(self, log):
        super(report_journal, self).__init__()
    	self.log = log


    def check(self, org,  date1, date2):
	err = ""
	date1 = mylib.ctod(date1)
	date2 = mylib.ctod(date2)

	if date1 is None:
		err = "Initial date is invalid"
	elif date2 is None:
		err = "Final date is invalid"
	elif org <= 0:
		err = "Please choose an organization"
	else:
		if date1 > date2:
			date1, date2 = date2, date1

		try:
			org = model.orgs.get(org)
		except notfound, e:
			err = "Organization does not exist (just deleted?)"
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e)+" (SQL Error) "

	return (err, org, date1, date2)


    def query(self, org, date1, date2):
	err = ""
	rows = []
	dbsum = crsum = ""
	dbsumn = crsumn = 0
	title = self.title

	err, rorg, ddate1, ddate2 = self.check(org, date1, date2)
	if err:
		return (title, rows, dbsum, crsum, err)

	title = self.title_mask % (rorg.id, rorg.name, mylib.dtoc(ddate1), mylib.dtoc(ddate2))

	mylib.setdefault(self, "date1", date1, "")
	mylib.setdefault(self, "date2", date2, "")
	mylib.setdefault(self, "org", org, 0)

	d = {}

	try:
		batch_open = model.cmos.get(1).batch_open
		
		blex = model.journal.select(sqlbuilder.AND( \
					  model.journal.q.day >= ddate1, model.journal.q.day <= ddate2, \
					  model.journal.q.cancelled <= 0, \
		 		   	  sqlbuilder.SQLConstant("org_id = %d" % rorg.id)), \
		 			  orderBy=(model.journal.q.day, model.journal.q.id) )

		lastday = ""
		for row in blex:
			if not row.batch.closed and not batch_open:
				continue
			value100 = int(row.value)
			d = {}
			rows.append(d)
			d["id"] = row.id
			d["batch"] = row.batch.id
			d["date"] = mylib.dtoc(row.day)
			d["code"] = row.account.code
			d["account_description"] = row.account.description
			if d["date"] == lastday:
				d["date"] = ""
			else:
				lastday = d["date"]
			d["description"] = row.description 
			if row.stdjournal.id > 1:
				d["description"] = row.stdjournal.description+u" "+d["description"]
			if value100 > 0:
				d["db"] = u""
				d["cr"] = mylib.monetary(value100)
				crsumn += value100
			else:
				d["db"] = mylib.monetary(-value100)
				d["cr"] = u""
				dbsumn -= value100
			d["ledger_params"] = {"org": rorg.id, "date1": date1, "date2": date2, "account": d["code"], "submitted": 1}
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e)+" (SQL Error) "

	dbsum = mylib.monetary(dbsumn)
	crsum = mylib.monetary(crsumn)

	return (title, rows, dbsum, crsum, err) 


    def select_tables(self):
	orgs = [{"id": 0, "name": "---"}]
	err = ""

	try:
		for row in model.orgs.select(orderBy=model.orgs.q.name):
			orgs.append({"id": row.id, "name": "%d - %s" % (row.id, row.name)})
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) + "(reading organizations)"

	return (orgs, err)

    def perm_org(self, d):
	return d["org"]

    @expose(template="luca.templates.report_journal")
    @identity.require(identity.not_anonymous())
    @validate(validators=validators)
    def index(self, org=0, date1="", date2="", submitted=0, **dummy):
	err = ""
	rows = []
	dbsum = crsum = ""
	title = self.title
	if submitted:
		try:
			grant = model.permission(self, "read", org, 0)
			title, rows, dbsum, crsum, err = self.query(org, date1, date2)
			if err:
				grant.fail(err)
			else:
				grant.done()
		except denied, e:
			err = e.args[0]
	
	orgs, err2 = self.select_tables()

	date1 = mylib.getdefault(self, "date1", date1, "")
	date2 = mylib.getdefault(self, "date2", date2, "")
	org = mylib.getdefault(self, "org", org, 0)

	if err or err2:
		flash(err or err2)
		err = err or err2

	return dict(title=title, org=org, date1=date1, date2=date2, orgs=orgs, rows=rows, dbsum=dbsum, crsum=crsum, tg_flash_class="errmsg", timestamp=mylib.timestamp(), lite=model.session()['lite'], err=err)


    @expose() #pragma: no cover
    def lite(self, **dummy): #pragma: no cover
	redirect("./") #pragma: no cover

