# -*- coding: UTF-8 -*-
"""Defines the tasks to be run by the scheduler."""

__all__ = ['add_all_tasks', 'added_tasks', 'run_startup_tasks']

__author__    = "Christopher Arndt"
__revision__  = "$Rev:$"
__date__      = "$Date:$"
__license__   = "LGPL"

import logging

import turbogears.scheduler

from turbogears import config

from luca.model import Currency, CurrencyRate

log = logging.getLogger("luca.scheduler")

def download_fx_rates():
    """Download currency exchange rates from Federal Reserve Bank New York."""

    currencies = Currency.select()
    log.info(u'Starting download of currency exchange rates.')
    for curr in currencies:
        if not curr.autoDownload:
            log.debug(u'Not downloading rates for %s.' % curr.code)
            continue
        if curr.code == config.get('luca.fxrates.key_currency', 'USD'):
            continue
        try:
            msgs = curr.download_rates()
        except Exception, e:
            log.warning(u'Error downloading rates for %s: %s' % (curr.code, e))
        else:
            for msg in msgs:
                log.debug(msg)
    else:
        log.info(u'Download of currency exchange rates completed.')

def download_fx_rates_startup():
    """Download currency exchange rates from FRBNY on startup."""

    if CurrencyRate.select(CurrencyRate.q.manualOverride == False).count() == 0:
        log.info(u'First time download of currency exchange rates.')
        download_fx_rates()

def import_currencies_startup():
    """Import currencies from data/CurrencyCodeList.xml on start-up."""

    if Currency.select().count() == 0:
        log.info(u'First time import of currencies.')
        from pkg_resources import Requirement, resource_filename
        from luca.commands.import_currencies import import_currencies

        try:
            filename = resource_filename(Requirement.parse("luca"),
              "data/CurrencyCodeList.xml")
        except:
            filename = "data/CurrencyCodeList.xml"
        import_currencies(filename)

# add tasks that should be run on application startup here
startup_tasks = [
    (import_currencies_startup, {}),
    (download_fx_rates_startup, {}),
]
# add interval tasks that should be added on application startup here
interval_tasks = {}
# add weekday tasks that should be added on application startup here
weekday_tasks = {
    'download_fx_rates': {
        'weekdays': config.get('luca.fxrates.download_task.weekdays',
            [1,2,3,4,5]),
        'timeonday': config.get('luca.fxrates.download_task.timeonday', (19,00)),
        'action': download_fx_rates
    }
}
# add monthday tasks that should be added on application startup here
monthday_tasks = {}

# modul global dict for holding all scheduler task objects
added_tasks = {}

def add_all_tasks():
    """Add all tasks in "default_tasks" to the scheduler."""

    global added_tasks
    for task, params in interval_tasks.items():
        added_tasks[task] = turbogears.scheduler.add_interval_task(
          taskname=task, **params)
    for task, params in weekday_tasks.items():
        added_tasks[task] = turbogears.scheduler.add_weekday_task(
          taskname=task, **params)
    for task, params in monthday_tasks.items():
        added_tasks[task] = turbogears.scheduler.add_monthday_task(
          taskname=task, **params)

def run_startup_tasks():
    """Run all tasks that should be run on application start-up."""

    for task, params in startup_tasks:
        task(**params)
