import cherrypy

import turbogears
from turbogears import controllers, expose, validate
from turbogears import identity, validators
from sqlobject import sqlbuilder

from luca.model import xgroup, conn, notfound, denied
from luca import model

from luca.simple_table import simple_table

class sys_groups(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
        	  "group_name": validators.UnicodeString(if_invalid=u""),
		  "display_name": validators.UnicodeString(if_invalid=u"")}
				 
    def setup(self):
	# special ACL that is accepted as valid permission
	self.vips = ["ADMIN"]

    	self.emptyrow_read = {"id": 0, "group_name": "", "display_name": ""}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [{}, {}, {}, {}]
	self.invalid_data = [("group_name", ""), ("display_name", "")]
	self.unique_cols = ["group_name"]
	self.foreign_keys = []
	self.test_order = 8
	self.test_flags = ('cant_remove_id_1',)
	self.test_bump = [ ('cmos', 'xgroup_version', 1) ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Group not found, probably it has just been deleted"
	self.show_msg_ajax = False
    	self.msg_added = "Group has been added"
    	self.msg_updated = "Group has been updated"
    	self.msg_removed = "Group has been removed sucessfully"
    	self.msg_validation = "Name, nickname and e-mail address must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Group can not be removed"
    	self.msg_cannotremove_admin = "The Administrator cannot be removed!"
    	self.msg_unique = "Make sure that group name is unique"
    	self.msg_legend_update = "System groups: editing group '%s'"
    	self.msg_legend_add = "System groups: adding new group"

    	self.table = xgroup
	self.streamlined_add = True

    def row_name(self, d_row):
	name = ""
	if d_row["id"] > 0:
		name = u"%s (id %d)" % (d_row["group_name"], d_row["id"])
	else:
		name = d_row["group_name"]
	return name

    def form_to_intermediate_ext(self, form_data, d):
   	d["group_name"] = d["group_name"][:16].strip()
	d["display_name"] = d["display_name"][:40].strip()
    
    def intermediate_validate(self, d):
	    err = ""
	    if not d["group_name"]:
		err = "Nickname must not be empty"
	    elif not d["display_name"]:
		err = "Group name must not be empty"
	    if d["id"] > 0:
		if self.table.select(sqlbuilder.AND(self.table.q.group_name == d["group_name"], self.table.q.id != d["id"])).count() > 0:
			err = self.msg_integrity + self.msg_unique
	    else:
		if self.table.select(self.table.q.group_name == d["group_name"]).count() > 0:
			err = self.msg_integrity + self.msg_unique
	    return err

    def data_legend(self, d):
	    return d["group_name"]
    
    def trigger_remove_pre(self, row):
	if row["id"] <= 1:
		return self.msg_cannotremove_admin
	return ""

    def trigger_update_post(self, row, id, new, form_data):
	    model.cmos.bump_version("xgroup")

    def trigger_remove_post(self, row):
	    model.cmos.bump_version("xgroup")

    def table_version(self, row):
	return model.cmos.get_version("xgroup")

# EXPOSED METHODS ########################################################


    @expose(template="luca.templates.sys_groups")
    def index(self):
        return self.base_index()

    @expose(template="luca.templates.sys_groups_lite")
    def lite(self, **garbage):
        return self.base_index()

    @expose("json")
    @validate(validators=validators)
    def remove(self, id, **garbage):
	    return self.base_remove(id)

    @expose()
    @validate(validators=validators)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    return self.base_fetch(id)

    @expose(template="luca.templates.sys_groups_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	    return self.base_lite_edit(id)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    return self.base_update(form_data)

    @expose(template="luca.templates.sys_groups_lite_form")
    @validate(validators=validators)
    def lite_commit(self,  **form_data):
	    return self.base_lite_commit(form_data)

