import cherrypy

import turbogears
from turbogears import controllers, redirect, flash
from turbogears import identity, validators, expose, validate

from luca.model import sqlerror, notfound, log, xuser, modules, other_modules, function_points_sql, function_points_human, denied

from sqlobject import sqlbuilder

from luca import mylib
from luca.simple_report import simple_report
import datetime
import time
import model

class sys_qlogs(simple_report):

# MODEL- AND LANGUAGE-SPECIFIC CODE & MESSAGES ######################################

    validators = {"period1": validators.Int(if_invalid=0),  \
		  "period2": validators.Int(if_invalid=0),  \
		  "user": validators.Int(if_invalid=0),  \
		  "module": validators.Int(if_invalid=0),  \
		  "function": validators.Int(if_invalid=0),  \
		  "failedop": validators.Int(if_invalid=0),  \
		  "page": validators.Int(if_invalid=0) }
				 
    def setup(self):
	   # special ACL that is accepted as valid permission
	   self.vips = ["ADMIN"]

 	   self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
           self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
           self.msg_validation = self.msg_validation_crash
	   self.msg_period = "Please select a non-null period"
	   self.myself = "Myself"
	   self.msg_nothing = "No log entries found"

	   self.valid_requests = [ {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	  			   {"period1": 0, "period2": 7, "user": 2, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	  			   {"period1": 0, "period2": 7, "user": 2, "module": 2, "function": 0, "failedop": 0, "page": 1}, \
	  			   {"period1": 0, "period2": 7, "user": 2, "module": 0, "function": 1, "failedop": 0, "page": 1}, \
	  			   {"period1": 0, "period2": 7, "user": 2, "module": 0, "function": 0, "failedop": 1, "page": 1}, \
	  			   {"period1": 0, "period2": 7, "user": 2, "module": 0, "function": 0, "failedop": 2, "page": 1}, \
	  			   {"period1": 6, "period2": 7, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	  			   {"period1": 0, "period2": 7, "user": 2, "module": 0, "function": 0, "failedop": 0, "page": 1} ]

	   self.invalid_requests = [ {"period1": 999, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	  			     {"period1": 7, "period2": 999, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	  			     { "period1": 7, "period2": 0, "user": 0, "module": 9999, "function": 0, "failedop": 0, "page": 1}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 999, "failedop": 0, "page": 1}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 999, "page": 1}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 999}, \
	  			     {"period1": 7, "period2": 0, "user": 999, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	                             {"period1": 999, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 0}, \
	  			     {"period1": 7, "period2": 999, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 0}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 9999, "function": 0, "failedop": 0, "page": 0}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 999, "failedop": 0, "page": 0}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 999, "page": 0}, \
	  			     {"period1": 7, "period2": 0, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 999}, \
	  			     {"period1": 7, "period2": 7, "user": 0, "module": 0, "function": 0, "failedop": 0, "page": 1}, \
	  			     {"period1": 7, "period2": 0, "user": 999, "module": 0, "function": 0, "failedop": 0, "page": 0} ]

	   # self.test_flags = ('anyone_index_allowed','admin_as_winner')
	   self.test_flags = ('anyone_index_allowed')

	   self.periods  = ["Now", "One hour ago", "Yesterday", "Last week", "A month ago", "A quarter ago", "A year ago", "From 1970"]
	   self.periodsN = [0, 3600, 86400*2, 86400*8, 86400*32, 86400*32*3, 86400*366, time.time()]

	   self.modules = ["(Any module)"] + modules + other_modules
	   self.modulesN = self.modules[:]

	   self.functions = function_points_human[:]
	   self.functionsN = function_points_sql[:]

	   self.failedops = ["I don't care", "Successful ops only", "Failed ops only"]
	   self.page_size = 150

    def __init__(self, log):
        super(sys_qlogs, self).__init__()
    	self.log = log

    def check(self, period1, period2, user, module, function, failedop, page):
	ok = period1 >= 0 and period1 < len(self.periods) and \
		period2 >= 0 and period2 < len(self.periods) and \
		module >= 0 and module < len(self.modules) and \
		function >= 0 and function < len(self.functions) and \
		failedop >= 0 and failedop < len(self.failedops) and \
		user >= 0 

	if not ok:
		return self.msg_validation

	if period1 == period2:
		return self.msg_period

	if user:
		try:
			xuser.get(user)
		except notfound:
			return "User not found"
		except sqlerror, e:  #pragma: no cover
			return self.sqlerr(e)+" (user table) "

	return ""



    def query(self, granted, period1, period2, user, module, function, failedop, page):
	prevpage = prevpage_url = nextpage = nextpage_url = None
	err = ""
	rows = []
	
	sql_period1 = self.periodsN[period1]
	sql_period2 = self.periodsN[period2]

	if sql_period1 < sql_period2:
		sql_period1, sql_period2 = (sql_period2, sql_period1)

	sql_period1 = datetime.datetime.fromtimestamp(time.time() - sql_period1)
	sql_period2 = datetime.datetime.fromtimestamp(time.time() - sql_period2)

	constraints = [log.q.tmstamp >= sql_period1, log.q.tmstamp <= sql_period2]

	if granted:
		if user:
			constraints.append(sqlbuilder.SQLConstant("user_id = %d" % user))
	else:
		constraints.append(sqlbuilder.SQLConstant("user_id = %d" % identity.current.user.id))

	if module:
		constraints.append(log.q.module == self.modulesN[module])

	if function:
		constraints.append(sqlbuilder.LIKE(log.q.function, self.functionsN[function]))

	if failedop == 1:
		constraints.append(log.q.err == "")
	elif failedop == 2:
		constraints.append(log.q.err != "")

	line = 0
	try:
		for row in log.select(sqlbuilder.AND(*constraints), orderBy=log.q.tmstamp).reversed():
			this_page = int(line / self.page_size) + 1
			if this_page == page:
				rows.append(row)
			line += 1
		if line == 0:
			err = self.msg_nothing
			totalpages = 1
			page = 1
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e)+" (SQL Error) "
		totalpages = 0
		page = 0

	totalpages = int(line / self.page_size) + 1

	if page > 1:
		prevpage_url = dict(period1=period1, period2=period2, user=user, module=module,
						function=function, failedop=failedop, page=page-1)
		prevpage = "%d" % (page - 1)

	if page < totalpages:
		nextpage_url = dict(period1=period1, period2=period2, user=user, module=module,
						function=function, failedop=failedop, page=page+1)
		nextpage = "%d" % (page + 1)

	return (rows, page, prevpage, prevpage_url, nextpage, nextpage_url, totalpages, err) 



    def select_tables(self, granted, period1, period2, user, module, function, failedop, page):
	# lists are copied here since if the lists were kept in the controller object,
	# multiple threads could clash

	periods1 = [ {"id": i, "name": x, "selected": None} for i, x in enumerate(self.periods) ]
	periods2 = [ {"id": i, "name": x, "selected": None} for i, x in enumerate(self.periods) ]
	functions = [ {"id": i, "name": x, "selected": None} for i, x in enumerate(self.functions) ]
	failedops = [ {"id": i, "name": x, "selected": None} for i, x in enumerate(self.failedops) ]
	modules = [ {"id": i, "name": x, "selected": None} for i, x in enumerate(self.modules) ]
	users = [ {"id": 0, "name": "(Any user)", "selected": None} ]
	err = ""

	if not page:
		# no submitted data, or submitted data has errors
		period2 = 0
		period1 = len(self.periods) - 1
		user = 0
		module = function = failedop = 0

	users[0]["selected"] = "1"
	if granted:
		try:
			for row in xuser.select(orderBy=xuser.q.user_name):
				users.append({"id": row.id, "name": row.user_name, "selected": None})
				if row.id == user:
					users[-1]["selected"] = 1
					users[0]["selected"] = None
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e) + "(reading Users table)"
	else:
		users[0]["name"] = self.myself

	try:
		periods1[period1]["selected"] = "1"
	except IndexError:  #pragma: no cover
		pass

	try: 
		periods2[period2]["selected"] = "1"
	except IndexError: #pragma: no cover
		pass

	try: 
		modules[module]["selected"] = "1"
	except IndexError: #pragma: no cover
		pass

	try: 
		functions[function]["selected"] = "1"
	except IndexError: #pragma: no cover
		pass

	try: 
		failedops[failedop]["selected"] = "1"
	except IndexError: #pragma: no cover
		pass

	return (periods1, periods2, users, modules, functions, failedops, err)


    @expose(template="luca.templates.sys_qlogs")
    @identity.require(identity.not_anonymous())
    @validate(validators=validators)
    def index(self, period1=0, period2=0, user=0, module=0, function=0, failedop=0, page=0, **dummy):

	try:
		grant = model.permission(self, "read", 0, model.permission.MASK_DONE | model.permission.MASK_DENIED)
	except denied, e:
		grant = None

	prevpage = prevpage_url = nextpage = nextpage_url = err = totalpages = None
	rows = []
	if page:
		# query has been submitted
		err = self.check(period1, period2, user, module,function, failedop, page)
		if err:
			page = 0
		else:
			rows, page, prevpage, prevpage_url, nextpage, nextpage_url, totalpages, err = self.query(grant, period1, period2, user, module, function, failedop, page)
			if err and err != self.msg_nothing: 
				# only happens in case of catastrophic SQLError exception 
				if grant: #pragma: no cover
					grant.fail(err)
			elif page == 1:
				# to avoid too much logging about the same query
				if grant:
					grant.done()
	
	# Gets select field lists
	periods1, periods2, users, modules, functions, failedops, err2 = self.select_tables(grant, period1, period2, \
				   user, module, function, failedop, page)
	if err or err2:
		flash(err or err2)
		err = err or err2

	return dict(periods1=periods1, periods2=periods2, users=users, modules=modules, 
			functions=functions, failedops=failedops, \
			rows=rows, prevpage=prevpage, nextpage=nextpage, prevpage_url=prevpage_url, \
			nextpage_url=nextpage_url, totalpages=totalpages, page=page, tg_flash_class="errmsg",
			err=err)

    @expose() #pragma: no cover
    def lite(self, **dummy): #pragma: no cover
	redirect("./") #pragma: no cover

