#!/usr/bin/env python

from xml.dom import minidom
from xml.dom import ext
import sys

global META

def getElementById(e, v):
     children = [x for x in e.childNodes if x.nodeType == e.ELEMENT_NODE]
     for x in children:
         if x.getAttribute('id') == v:
             return x
         if x.childNodes:
             t = getElementById(x, v)
             if t:
                 return t

def getElementByAttrName(e, v):
     children = [x for x in e.childNodes if x.nodeType == e.ELEMENT_NODE]
     for x in children:
         if x.getAttribute(v):
             return x
         if x.childNodes:
             t = getElementByAttrName(x, v)
             if t:
                 return t
     return None

def fillText(e, s):
	while e.childNodes:
		delete(e.childNodes[0])
	e.appendChild(META.createTextNode(s))

def delete(el):
	el.parentNode.removeChild(el)

def removeByTemplate(e, n):
     children = [x for x in e.childNodes if x.nodeType == e.ELEMENT_NODE]
     for x in children:
         y = x.getAttribute('template')
         if y == n:
            delete(x)
	 elif x.childNodes:
               t = removeByTemplate(x, n)

def removeAttribute(e, a):
     children = [x for x in e.childNodes if x.nodeType == e.ELEMENT_NODE]
     for x in children:
         if x.getAttribute(a):
	     x.removeAttribute(a)
         removeAttribute(x, a)

def replaceMeta(doc_tag, meta_id, document, manipulate_new_node = None):
	templates = getElementById(META, meta_id)
	for pivot in document.getElementsByTagName(doc_tag):
		if templates:
			for t in templates.childNodes:
				template = t.cloneNode(deep=True)
				pivot.parentNode.insertBefore(template, pivot)
				if manipulate_new_node:
					manipulate_new_node(template, pivot)
		delete(pivot)

def manipulate_script(template, pivot):
	if template.nodeType == template.ELEMENT_NODE:
		file = pivot.getAttribute('file')
		fqdn_file = template.getAttribute('SRC')
		fqdn_file = fqdn_file.replace('#file#', file)
		template.setAttribute('SRC', fqdn_file)

def manipulate_title(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == 'title':
		title = pivot.getAttribute('index')
		fillText(template, title)

def manipulate_title_form(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == 'title':
		title = pivot.getAttribute('form')
		fillText(template, title)

def manipulate_header(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == 'h3':
		title = pivot.getAttribute('text')
		fillText(template, title)

def manipulate_invcol(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "td":
		template = template.getElementsByTagName("span")[0]
		sid = pivot.getAttribute('sid')
		protoid = template.getAttribute('id')
		template.setAttribute('id', protoid.replace('#sid#', sid))

def manipulate_invlink(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "td":
		template = template.getElementsByTagName("a")[0]
		href = pivot.getAttribute('href')
		protohref = template.getAttribute('href')
		text = pivot.getAttribute('text')
		template.setAttribute('href', protohref.replace('#href#', href))
		fillText(template, text)

def manipulate_col(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "td":
		template = template.getElementsByTagName("span")[0]
		sid = pivot.getAttribute('sid')
		key = pivot.getAttribute('key')
		col = pivot.getAttribute('col')
		if sid and key:
			protoid = template.getAttribute('id')
			template.setAttribute('id', protoid.replace('#sid#', sid).replace('#key#', key))
		else:
			template.removeAttribute('id')
		template.setAttribute('py:content', col)

def manipulate_link_ajax(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "td":
		template = template.getElementsByTagName("a")[0]
		ajax_call = pivot.getAttribute('ajax')
		keys = [pivot.getAttribute('key3'), pivot.getAttribute('key2'), pivot.getAttribute('key')]
		keys = [('${%s}' % x) for x in keys if x]
		keys = ", ".join(keys)
		protohref = template.getAttribute('href')
		template.setAttribute('href', protohref.replace('#ajax#', ajax_call).replace('#keys#', keys))

def manipulate_link_lite(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "td":
		for template in template.getElementsByTagName("a"):
			keys = [ ('grandmother', pivot.getAttribute('key3')), \
				 ('mother', pivot.getAttribute('key2')), \
				 ('id', pivot.getAttribute('key')) ]
			keys = [('%s=${%s}' % x) for x in keys if x[1]]
			keys = "&amp".join(keys)
			protohref = template.getAttribute('href')
			template.setAttribute('href', protohref.replace('#keys#', keys))

def manipulate_input(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "tr":
		hdr = template.getElementsByTagName("td")[0]
		header = pivot.getAttribute('header')
		fillText(hdr, header)

		field = template.getElementsByTagName("input")[0]
		for attr in pivot.attributes.keys():
			if attr != 'header':
				field.setAttribute(attr, pivot.getAttribute(attr))

def manipulate_textarea(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "tr":
		hdr = template.getElementsByTagName("td")[0]
		header = pivot.getAttribute('header')
		fillText(hdr, header)

		field = template.getElementsByTagName("textarea")[0]
		for attr in pivot.attributes.keys():
			if attr != 'header':
				field.setAttribute(attr, pivot.getAttribute(attr))

def manipulate_select(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == "tr":
		hdr = template.getElementsByTagName("td")[0]
		header = pivot.getAttribute('header')
		fillText(hdr, header)

		field = template.getElementsByTagName("select")[0]
		name = pivot.getAttribute('name')
		proto = field.getAttribute('name')
		field.setAttribute('name', proto.replace('#name#', name))

		field = field.getElementsByTagName("option")[0]

		value = pivot.getAttribute('value')
		iterator = pivot.getAttribute('iterator')
		list = pivot.getAttribute('list')
		preselected  = pivot.getAttribute('preselected')

		proto = field.getAttribute('value')
		field.setAttribute('value', proto.replace('#value#', value))
		proto = field.getAttribute('py:for')
		field.setAttribute('py:for', proto.replace('#iterator#', iterator).replace('#list#', list))
		proto = field.getAttribute('selected')
		field.setAttribute('selected', proto.replace('#preselected#', preselected).replace('#value#', value))

		field = field.getElementsByTagName("span")[0]
		text = pivot.getAttribute('text')
		proto = field.getAttribute('py:content')
		field.setAttribute('py:content', proto.replace('#text#', text))

def manipulate_linkindex_lite_form(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == 'a':
		text = pivot.getAttribute('text')
		fillText(template, text)

def manipulate_addnew_ajax(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == 'div':
		template = template.getElementsByTagName('a')[0]
		text = pivot.getAttribute('text')
		fillText(template, text)

def manipulate_addnew_lite(template, pivot):
	if template.nodeType == template.ELEMENT_NODE and template.tagName == 'p':
		template = template.getElementsByTagName('a')[0]
		text = pivot.getAttribute('text')
		fillText(template, text)
		keys = [ ('grandmother', pivot.getAttribute('key3')), \
			 ('mother', pivot.getAttribute('key2')) ]
		keys = [('%s=${%s}' % x) for x in keys if x[1]]
		keys += [ "id=0" ]
		keys = "&amp".join(keys)
		protohref = template.getAttribute('href')
		template.setAttribute('href', protohref.replace('#keys#', keys))

if len(sys.argv) != 2:
	print "Usage: %s basename" % sys.argv[0]

basename = sys.argv[1]

META = minidom.parse("META.yml")
ajax = minidom.parse("%s.yml" % basename)
lite = minidom.parse("%s.yml" % basename)
lite_form = minidom.parse("%s.yml" % basename)

# remove sections that obviously do not belong to documents
for id in ('formdiv','newrow_invisible'):
	el = getElementById(lite, id)
	delete(el)

for id in ('tabledata',):
	el = getElementById(lite_form, id)
	delete(el)

# process elements that are specifically target to one template
removeByTemplate(ajax, 'LITE')
removeByTemplate(ajax, 'LITEFORM')
removeByTemplate(lite, 'AJAX')
removeByTemplate(lite, 'LITEFORM')
removeByTemplate(lite_form, 'AJAX')
removeByTemplate(lite_form, 'LITE')

# remove the 'template' attribute since it has been already processed
for d in (ajax, lite, lite_form):
	removeAttribute(d, 'template')

el = lite_form.getElementsByTagName('form')[0]
el.setAttribute('method', 'post')
el.setAttribute('action', 'lite_commit')

el = ajax.getElementsByTagName('form')[0]
el.setAttribute('method', 'get')
el.setAttribute('onsubmit', 'return ajax_submit()')
el = ajax.getElementsByTagName('body')[0]
el.setAttribute('onLoad', 'ajax_setup()')
el.setAttribute('onsubmit', 'return false')

el = getElementById(lite_form, 'formdiv')
el.setAttribute('class', 'shown')
el = getElementById(ajax, 'formdiv')
el.setAttribute('class', 'hidden')

# replace meta tags by the expected contents
replaceMeta('META_FORMFOOTER', 'formfooter_ajax', ajax)
replaceMeta('META_FORMFOOTER', 'formfooter_lite', lite_form)

replaceMeta('META_FORMFOOTER_NOREMOVE', 'formfooter_noremove_ajax', ajax)
replaceMeta('META_FORMFOOTER_NOREMOVE', 'formfooter_lite', lite_form)

replaceMeta('META_FORMHEADER', 'formheader_ajax', ajax)
replaceMeta('META_FORMHEADER', 'formheader_lite', lite_form)

replaceMeta('META_SCRIPT', 'script_lite', lite)
replaceMeta('META_SCRIPT', 'script_lite', lite_form)
replaceMeta('META_SCRIPT', 'script_ajax', ajax, manipulate_script)

replaceMeta('META_TITLE', 'title_index', lite, manipulate_title)
replaceMeta('META_TITLE', 'title_index', ajax, manipulate_title)
replaceMeta('META_TITLE', 'title_form', lite_form, manipulate_title_form)

replaceMeta('META_HEADER', 'header_index', lite, manipulate_header)
replaceMeta('META_HEADER', 'header_index', ajax, manipulate_header)
replaceMeta('META_HEADER', 'header_form', lite_form)

replaceMeta('META_INVCOL', 'invcol', ajax, manipulate_invcol)
replaceMeta('META_INVLINK', 'invlink', ajax, manipulate_invlink)

replaceMeta('META_COL', 'col', ajax, manipulate_col)
replaceMeta('META_COL', 'col', lite, manipulate_col)

replaceMeta('META_LINK', 'link_ajax', ajax, manipulate_link_ajax)
replaceMeta('META_LINK', 'link_lite', lite, manipulate_link_lite)

replaceMeta('META_LINK_NOREMOVE', 'link_ajax', ajax, manipulate_link_ajax)
replaceMeta('META_LINK_NOREMOVE', 'link_noremove_lite', lite, manipulate_link_lite)

replaceMeta('META_INPUT', 'input', ajax, manipulate_input)
replaceMeta('META_INPUT', 'input', lite_form, manipulate_input)

replaceMeta('META_TEXTAREA', 'textarea', ajax, manipulate_textarea)
replaceMeta('META_TEXTAREA', 'textarea', lite_form, manipulate_textarea)

replaceMeta('META_SELECT', 'select_ajax', ajax, manipulate_select)
replaceMeta('META_SELECT', 'select_lite_form', lite_form, manipulate_select)

replaceMeta('META_LINKINDEX', 'linkindex_ajax', ajax)
replaceMeta('META_LINKINDEX', 'linkindex_lite', lite)
replaceMeta('META_LINKINDEX', 'linkindex_lite_form', lite_form, manipulate_linkindex_lite_form)

replaceMeta('META_ADDNEW', 'addnew_ajax', ajax, manipulate_addnew_ajax)
replaceMeta('META_ADDNEW', 'addnew_lite', lite, manipulate_addnew_lite)
replaceMeta('META_ADDNEW', 'addnew_lite_form', lite_form)

# lite forms' input fields have initial values prefilled by controller, not by javascript
for f in ('input', 'textarea'):
	for el in lite_form.getElementsByTagName(f):
		col_name = el.getAttribute('name')
		el.setAttribute('value', '${%s}' % col_name)

# write down the result
ext.PrettyPrint(ajax, open("%s.xml" % basename, "w"))
ext.PrettyPrint(lite, open("%s_lite.xml" % basename, "w"))
ext.PrettyPrint(lite_form, open("%s_lite_form.xml" % basename, "w"))

