## -*- coding: utf-8 -*-

# Copyright (c) 2005 INdT - Instituto Nokia de Tecnologia, Elvis Pfutzenreuter.
# All rights reserved. This software is released under the LGPL licence.

import appuifw
from mylib import *
import e32
from key_codes import *

BROWSE = 1
STUCK = 2
ADD = 3
ALT = 4
REMOVE = 5
SYNCING = 6

class face_s60:
	def left(self):
		self.delta = -1
		self.lock.signal()
	
	def right(self):
		self.delta = 1
		self.lock.signal()
	
		self.lock.signal()
	def first(self):
		self.delta = -999999999
		self.lock.signal()
	
	def last(self):
		self.delta = 999999999
		self.lock.signal()
	
	def add(self):
		self.mode = ADD
		self.title(u"LucaMobile: new journal")

		addform = appuifw.Form( [ \
				(u'DB', 'combo', self.ui_accounts_list()), \
				(u'CR', 'combo', self.ui_accounts_list()), \
				(u'Date', 'date', now()), \
				(u'$', 'number', 0), \
				(u'Desc', 'text', u""), \
				] )

		addform.save_hook = self.add_save_hook
		addform.flags = appuifw.FFormEditModeOnly

		while 1:
			self.saved = False
			addform.execute()

			if not self.saved:
				break

			# subscription: [field][data][[index, if combo]]
			dbindex = addform[0][2][1]
			crindex = addform[1][2][1]
			date = addform[2][2]
			value = addform[3][2]
			description = addform[4][2]
			description = description.strip()[0:60]

			err = self.err_journal(1, dbindex, crindex, date, value, description)
			if err:
				self.msg(err)
			else:
				break

		if self.saved: 
			new_journals = self.gen_journal(1, dbindex, crindex, date, value, description)
	
			for new_journal in new_journals:
				try:
					self.journal_position = self.repo_journal.append(new_journal)

				except DatamodelError, e:
					self.err(u"Error: %s" % unicode(e))
					return

			if len(new_journals) > 1:
				self.msg(u"Journal entries added.")
			else:
				self.msg(u"Journal entry added.")

		del addform
	
		self.title()
		self.lock.signal()
	
	def add_save_hook(self, form):
		self.saved = True
		return True

	def remove(self):
		if self.mode != BROWSE:
			self.err(u"No journal entry to remove!")
			return
	
		self.mode = REMOVE

		if appuifw.query(u"Confirm journal removal?", 'query'):
			try:
				self.repo_journal.remove(self.present_journal["key"])
				self.msg(u"Journal entry removed.")
			except DatamodelError, e:
				self.err(u"Error: %s" % unicode(e))

		self.lock.signal()
	
	def alt(self):
		if self.mode != BROWSE:
			self.err(u"No journal entry to update!")
			return

		self.mode = ALT
		self.saved = False

		self.title(u"LucaMobile: update journal")

		if self.present_journal["value"] < 0: 
			db = self.present_journal["account"]
			cr = None
		else:
			cr = self.present_journal["account"]
			db = None

		altform = appuifw.Form( [ \
				(u'DB', 'combo', self.ui_accounts_list(db)), \
				(u'CR', 'combo', self.ui_accounts_list(cr)), \
				(u'Date', 'date', self.present_journal["date"]), \
				(u'$', 'number', abs(int(self.present_journal["value"]))), \
				(u'Desc', 'text', self.present_journal["description"]), \
				] )

		altform.save_hook = self.alt_save_hook
		altform.flags = appuifw.FFormEditModeOnly

		while 1:
			self.saved = False
			altform.execute()

			if not self.saved:
				break

			# subscription: [field][data][[index, if combo]]
			dbindex = altform[0][2][1]
			crindex = altform[1][2][1]
			date = altform[2][2] 
			value = altform[3][2]
			description = altform[4][2]
			description = description.strip()[0:60]

			err = self.err_journal(0, dbindex, crindex, date, value, description)
			if err:
				self.msg(err)
			else:
				break
	
		if self.saved:
			changed_journal = self.gen_journal(0, dbindex, crindex, date, value, 
								description)[0]

			changed_journal["key"] = self.present_journal["key"]
			try:
				self.journal_position = self.repo_journal.update(changed_journal)
				# unneeded, will show the updated entry at screen
				# self.msg(u"Journal entry updated.")
	
			except DatamodelError, e:
				self.err(u"Error: %s" % unicode(e))

		del altform
		
		self.title()
		self.lock.signal()

	def alt_save_hook(self, form):
		self.saved = True
		return True

	def init(self):
		self.init_handler()
		self.ui_accounts_persist = None
		self.ui_accountcodes_persist = None
		self.msg(u"Database is empty")
		self.lock.signal()

	def sync(self):
		if self.mode != STUCK and self.mode != BROWSE:
			return

		diff = self.repo_journal.balance()
		if diff:
			self.err(u"Journal not balanced, difference is %.2f" % (diff / 100.0))
			return

		self.mode = SYNCING
		self.title(u"LucaMobile: syncing...")

		ret, err = self.sync_handler(1, "")
		if err:
			self.err(ret)
		else:
			self.msg(u"Synchronization complete")

		self.ui_accounts_persist = None
		self.ui_accountcodes_persist = None
	
		self.title()
		self.lock.signal()

	def refresh_journal(self, delta = 0):
		self.mode = BROWSE
		self.journal_position += delta

		if self.journal_position < 0:
			self.journal_position = 0
		if self.journal_position >= self.repo_journal.count():
			self.journal_position = self.repo_journal.count() - 1

		if self.journal_position >= 0:
			self.present_journal = self.repo_journal.get_by_pos(self.journal_position)
			self.show_journal()
		else:
			self.mode = STUCK
			appuifw.app.body.set(u"\r\n(no journal entries)\r\n")

	# General event/window handling section

	def title(self, s = u"LucaMobile"):
		appuifw.app.title = s
		# shows immediately
		e32.ao_yield()

	def msg(self, s, type="info"):
		appuifw.note(s, type)

	def err(self, s):
		self.msg(s, type="error")

	# Data manipulation section

	def err_journal(self, is_add, dbindex, crindex, date, value, description):
		if dbindex and crindex and not is_add:
			return u"Choose either debit or credit account, not both"

		if not dbindex and not crindex:
			return u"Choose either debit or credit account"

		if dbindex == crindex:
			return u"Debit and credit accounts must be different"
		
		if int(value) <= 0:
			return u"Value can not be zero"

		if not description:
			return u"Description is empty"

		if is_add and abs((now() - date)) > 28*DAYS:
			return u"Date too far apart from today"
		elif not is_add and abs((now() - date)) > 28*DAYS:
			return u"Date too far apart from today"

		return None

	def gen_journal(self, is_add, dbindex, crindex, date, value, description):
		ved = []
		tmstamp = now()
		
		if dbindex:
			ed = {}
			ed["type"] = u"journalentry"
			ed["description"] = description
			ed["account"] = self.ui_accountcodes_persist[dbindex]
			ed["date"] = date
			ed["value"] = -value
			ed["stdjournal"] = u'1' # by now
	                ed["timestamp"] = tmstamp
			ved.append(ed)
		
		if crindex:
			ed = {}
			ed["type"] = u"journalentry"
			ed["description"] = description
			ed["account"] = self.ui_accountcodes_persist[crindex]
			ed["date"] = date
			ed["value"] = value
			ed["stdjournal"] = u'1' # by now
	                ed["timestamp"] = tmstamp+1
			ved.append(ed)

		if not is_add and len(ved) > 1:
			raise Exception(u"is_add == 0 and ved > 1")

		return ved
	
	def show_journal(self):
		de = self.present_journal

		appuifw.app.body.clear()

		txt = u"Journal entry #%d\r\n" % (self.journal_position + 1)
		appuifw.app.body.color = 0x0000FF
		appuifw.app.body.add(txt)

		txt = dtoc(de["date"])

		if de["value"] < 0:
			# debit is green
			appuifw.app.body.color = 0x00C000
			# Bleh, % has higher precedence than /
			txt += u" $%0.2f DB\r\n" % (abs(de["value"])/100.0)
		else:
			# credit is red
			appuifw.app.body.color = 0xFF0000
			# Bleh, % has higher precedence than /
			txt += u" $%0.2f CR\r\n" % (abs(de["value"])/100.0)


		appuifw.app.body.add(txt)

		txt = self.repo_accounts.get_record(de["account"])["description"] + u"\r\n\r\n"
		appuifw.app.body.color = 0x0000FF
		appuifw.app.body.add(txt)

		txt = de["description"]
		appuifw.app.body.color = 0xA0A000
		appuifw.app.body.add(txt)

	def ui_accounts_list(self, def_account = None):
		if not self.ui_accounts_persist:
			l = self.repo_accounts.get_list()
			self.ui_accounts_persist = [u"--"]
			self.ui_accountcodes_persist = [u"--"]
			for account_reg in l:
				self.ui_accountcodes_persist.append(account_reg[0])
				self.ui_accounts_persist.append(account_reg[1])
		j = 0
		if def_account:
			j = self.ui_accountcodes_persist.index(def_account)

		return (self.ui_accounts_persist, j)

	# Window creation section

	def __init__(self, repo_journal, repo_accounts, sync_handler, init_handler):
		self.lock = e32.Ao_lock()

		self.repo_journal = repo_journal
		self.repo_accounts = repo_accounts
		self.sync_handler = sync_handler
		self.init_handler = init_handler

		self.ui_accounts_persist = None
		self.ui_accountcodes_persist = None

		self.mode = BROWSE
		self.journal_position = 0

		self.exit = 0
		self.delta = 0

		self.old_title = appuifw.app.title

		self.title()
		appuifw.app.screen = 'normal'  # large, full
		appuifw.app.exit_key_handler = self.destroy

		appuifw.app.menu = [ 	(u"Add", self.add), 
					(u"Update", self.alt), 
					(u"Remove", self.remove), 
					(u"--", self.dummy), 
					(u"Sync", self.sync), 
					(u"Exit", self.destroy),
					(u"--", self.dummy), 
					(u"First", self.first), 
					(u"Previous", self.left), 
					(u"Next", self.right), 
					(u"Last", self.last),
					(u"--", self.dummy),
					(u"Init database!", self.init) ]

		appuifw.app.body = appuifw.Text()
		appuifw.app.body.bind(EKeyLeftArrow, self.first)
		appuifw.app.body.bind(EKeyRightArrow, self.last)
		appuifw.app.body.bind(EKeyDownArrow, self.left)
		appuifw.app.body.bind(EKeyUpArrow, self.right)
		appuifw.app.body.bind(EKeySelect, self.add)
		appuifw.app.body.bind(EKeyBackspace, self.alt)

	def dummy(self):
		self.lock.signal()

	def destroy(self):
		self.exit = 1
		self.lock.signal()
	
	def main(self):
		while not self.exit:
			self.refresh_journal(self.delta)
			self.delta = 0
			self.lock.wait()

		appuifw.app.exit_key_handler = None
		self.title(self.old_title)
		appuifw.menu = None

		appuifw.app.body = None

