# Copyright (c) 2005, 2006 INdT - Instituto Nokia de Tecnologia, Elvis Pfutzenreuter.
# All rights reserved. This software is released under the LGPL licence.

from exceptions import StandardError
import time

DAYS = 86400

INDEX = xrange(999999999)

def timezone():
	tz = 0
	if time.__dict__.has_key("timezone"):
		tz = time.timezone
	return tz

def today():
	n = now()
	return n - (n % DAYS) + 1

def now():
	return time.time() - timezone()

def dtoymd(ndate):
	l = time.gmtime(ndate)
	return (l[0], l[1], l[2])

def dtohms(ndate):
	l = time.gmtime(ndate)
	return (l[3], l[4], l[5])

def dtoc(ndate):
	return u"%04d/%02d/%02d" % dtoymd(ndate)

def dtotime(ntime):
	return u"%02d:%02d:%02d" % dtohms(ntime)

def __check_date(year, month, day):
	if month > 12 or month < 1:
		return False

	months = [None, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
	leap = 0

	if month == 2:
		max_day = 28
		if (year % 4 == 0) and ((year % 100 != 0) or (year % 400 == 0)):
			leap = 1
			months[2] = 29

	if day > months[month] or day < 1:
		return False

	if year < 0 or year > 5000:
		return False
	
	return True

def ctod(sdate):
	'''Checks year/month/day and returns a float. Date must be in the
	   [yy]yy/mm/dd format.'''

	if not isinstance(sdate, unicode):
		raise Exception("Hey, check_date wants Unicode strings!")

	if len(sdate) < 5:
		return None

	sdate = sdate.replace(u'-', u'/')
	sdate = sdate.replace(u'.', u'/')
	sdate = sdate.replace(u',', u'/')

	dparams = sdate.split(u'/')
	if len(dparams) != 3:
		return None

	try:
		y = int(dparams[0])
		m = int(dparams[1])
		d = int(dparams[2])

	except ValueError:
		return None

	if y < 100:
		y += 2000

	if not __check_date(y, m, d):
		return None
	
	# makes date with 00:00:01AM time
	return time.mktime((y, m, d, 0, 0, 1, 0, 0, 0)) - timezone()

class MyException(StandardError):
	pass
# 	def __init__(self, args):
# 		self.args = args
 	def __unicode__(self):
		return u" ".join(map(unicode, self.args))
 	def __str__(self):
		return " ".join(map(str, self.args))
 	def __repr__(self):
 		return self.__str__()

# "Normal" exceptions that can occur during non-buggy program operation

class CommError(MyException):
	pass

class CommError_Idle(CommError):
	pass

class DatamodelError(MyException):
	pass
